# SDRAM controller for the Colorlight 5a-75b


from amaranth import Module, Cat, Signal, Assert, unsigned
from amaranth.build import Platform
from amaranth.lib import wiring, data, enum
from amaranth.lib.wiring import In, Out


# RAM Specs:
# 4 banks, 2048 rows, 256 columns.
# 11 row bits, 8 column bits.
# word size = 32
# 8 megabytes data.


sdram_control_layout = data.StructLayout(
    {
        "nCS": 1,
        "nRAS": 1,
        "nCAS": 1,
        "nWE": 1,
    }
)



class _WriteBurstLength(enum.Enum, shape=1):
    """MRS field"""
    BURST = 0
    SINGLE_BIT = 1

class _TestMode(enum.Enum, shape=2):
    """ The "test mode" of the sdram. This is always zero pretty much"""
    MODE_REGISTER_SET = 0
    RESERVED0 = 1
    RESERVED1 = 2
    RESERVED2 = 3

class _CASLatency(enum.Enum, shape=3):
    """ How many cycles of latency for the column address select to complete """
    CYCL2 = 2
    CYCL3 = 3

class _BurstType(enum.Enum, shape=1):
    SEQUENTIAL = 0
    INTERLEAVED = 1

class _BurstLength(enum.IntEnum, shape=3):
    """ The size of the burst """
    SINGLE = 0
    DUAL = 1
    QUAD = 2
    OCT = 3
    FULL_PAGE = 7


class _Command(enum.Enum):
    """ Command set for SDRAM """
    MRS_WRITE = 0
    ACTIVATE = 1
    PRECHARGE = 2
    WRITE = 3
    READ = 4
    CBR = 5 # auto refresh
    SELF_REFRESH = 6
    BRST_STOP = 7
    NOP = 8


